import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:scrobbles/common/config/screen.dart';
import 'package:scrobbles/common/cubit/nav/nav_cubit_pages.dart';
import 'package:scrobbles/common/cubit/nav/nav_cubit_screens.dart';

import 'package:scrobbles/cubit/activity/data_counts_by_day_cubit.dart';
import 'package:scrobbles/cubit/activity/data_counts_by_hour_cubit.dart';
import 'package:scrobbles/cubit/activity/data_discoveries_cubit.dart';
import 'package:scrobbles/cubit/activity/data_heatmap_cubit.dart';
import 'package:scrobbles/cubit/activity/data_new_artists_cubit.dart';
import 'package:scrobbles/cubit/activity/data_new_tracks_cubit.dart';
import 'package:scrobbles/cubit/activity/data_statistics_global_cubit.dart';
import 'package:scrobbles/cubit/activity/data_statistics_recent_cubit.dart';
import 'package:scrobbles/cubit/activity/data_timeline_cubit.dart';
import 'package:scrobbles/cubit/activity/data_top_artists_cubit.dart';

class GlobalAppBar extends StatelessWidget implements PreferredSizeWidget {
  const GlobalAppBar({super.key});

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<NavCubitScreen, int>(
      builder: (BuildContext context, int screenIndex) {
        final List<Widget> menuActions = [];

        if (screenIndex == Screen.indexActivity) {
          // go to Settings page
          menuActions.add(IconButton(
            onPressed: () {
              BlocProvider.of<NavCubitScreen>(context).goToScreenSettings();
            },
            icon: Screen.screenSettings.icon,
          ));

          // go to About page
          menuActions.add(IconButton(
            onPressed: () {
              BlocProvider.of<NavCubitScreen>(context).goToScreenAbout();
            },
            icon: Screen.screenAbout.icon,
          ));

          // refresh data
          menuActions.add(IconButton(
            onPressed: () {
              BlocProvider.of<DataCountsByDayCubit>(context).refresh(context);
              BlocProvider.of<DataCountsByHourCubit>(context).refresh(context);
              BlocProvider.of<DataDiscoveriesCubit>(context).refresh(context);
              BlocProvider.of<DataHeatmapCubit>(context).refresh(context);
              BlocProvider.of<DataNewArtistsCubit>(context).refresh(context);
              BlocProvider.of<DataNewTracksCubit>(context).refresh(context);
              BlocProvider.of<DataStatisticsGlobalCubit>(context).refresh(context);
              BlocProvider.of<DataStatisticsRecentCubit>(context).refresh(context);
              BlocProvider.of<DataTimelineCubit>(context).refresh(context);
              BlocProvider.of<DataTopArtistsCubit>(context).refresh(context);
            },
            icon: const Icon(UniconsSolid.refresh),
          ));
        } else {
          // back to Home page
          menuActions.add(IconButton(
            onPressed: () {
              BlocProvider.of<NavCubitScreen>(context).goToScreenActivity();
              BlocProvider.of<NavCubitPage>(context).goToPageHome();
            },
            icon: Screen.screenActivity.icon,
          ));
        }

        return AppBar(
          title: const AppHeader(text: 'app_name'),
          actions: menuActions,
        );
      },
    );
  }

  @override
  Size get preferredSize => const Size.fromHeight(50);
}
