import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:scrobbles/config/default_activity_settings.dart';
import 'package:scrobbles/models/settings/settings_activity.dart';

part 'settings_activity_state.dart';

class ActivitySettingsCubit extends HydratedCubit<ActivitySettingsState> {
  ActivitySettingsCubit()
      : super(ActivitySettingsState(settings: ActivitySettings.createDefault()));

  void setValues({
    String? dashboardType,
  }) {
    emit(
      ActivitySettingsState(
        settings: ActivitySettings(
          dashboardType: dashboardType ?? state.settings.dashboardType,
        ),
      ),
    );
  }

  String getParameterValue(String code) {
    switch (code) {
      case DefaultActivitySettings.parameterCodeDashboardType:
        return ActivitySettings.getDashboardTypeValueFromUnsafe(state.settings.dashboardType);
    }

    return '';
  }

  void setParameterValue(String code, String value) {
    final String dashboardType = code == DefaultActivitySettings.parameterCodeDashboardType
        ? value
        : getParameterValue(DefaultActivitySettings.parameterCodeDashboardType);

    setValues(
      dashboardType: dashboardType,
    );
  }

  @override
  ActivitySettingsState? fromJson(Map<String, dynamic> json) {
    final String dashboardType =
        json[DefaultActivitySettings.parameterCodeDashboardType] as String;

    return ActivitySettingsState(
      settings: ActivitySettings(
        dashboardType: dashboardType,
      ),
    );
  }

  @override
  Map<String, dynamic>? toJson(ActivitySettingsState state) {
    return <String, dynamic>{
      DefaultActivitySettings.parameterCodeDashboardType: state.settings.dashboardType,
    };
  }
}
